#!/usr/bin/env python3
# -*- coding: utf-8; py-indent-offset: 4 -*-


#  +-- bmSearch ------------------------------------------------------------------+
#  |                                                                              |
#  |  _                 _____                          _                          |
#  | | |               / ____|                        | |                         | 
#  | | |___  _ __ ___ | |____    ___  ____   _ _  ___ | |__                       |
#  | |  _  \| '_ ` _ \ \___  \  / _ \|___ \ | `_|/ __||  _  \                     | 
#  | | |_| || | | | | |____|  ||  __// __  || | | |__ | | | |                     |
#  | |_____/|_| |_| |_|______/  \___|\____/ |_|  \___||_| |_|                     |
#  |                                                                              |
#  |                                                                              |
#  +------------------------------------------------------------------------------+
#
# * ----------------------------------------------------------------------------------------*
# * @author  : Pascal SALAUN <pascal.salaun-AT-bm-monitor.org>                              *
# * @Website : http://www.bm-monitor.org <http://www.bm-monitor.org>                        *
# * @license : GNU Affero General Public License Version 3 <http://www.gnu.org/licenses>    *
# *                                                                                         *
# * bmSearch is free software;  you can redistribute it and/or modify it under the          *
# *  terms of the  license mentionned above.                                                *
# * bmSearch is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY    * 
# * ----------------------------------------------------------------------------------------*

# Native Python Lib/Class 
from collections import defaultdict
import json
import requests
import time


# bmSearch Class
from bmDate import *
import config as CFG

from logger import *
backoffice = logger().setLogger().getChild('requestToELK')

class requestToELK:
    """
    requestToELK:
    Class to send HTTP request (get/post) to an ELK platform
    """


    def __init__(self):
        """
            __init__ :
                Set default headers
                Set a (dict) dates from bmDate
        """
        
        self.params = defaultdict(dict)
        
        self.headers = {'Content-Type': "application/json; charset=utf-8", 'Accept': "application/json"}
        self.params['headers'] = self.headers
        self.pool_connections = 5
        self.pool_maxsize     = 10
        self.max_retries      = 3
        self.pool_block       = False
        self.APIURL           = CFG.APIURL
        
        if hasattr(CFG,'user') and hasattr(CFG,'password'):
            from requests.auth import HTTPBasicAuth
            self.params['auth'] = HTTPBasicAuth(CFG.user, CFG.password)
        
        if hasattr(CFG,'verifyPeer'):
            self.params['verify'] = CFG.verifyPeer
        
        if hasattr(CFG,'serverCert'):
            self.params['cert'] = CFG.serverCert

        if hasattr(CFG,'pool_connections'):
            self.pool_connections = CFG.pool_connections

        if hasattr(CFG,'pool_maxsize'):
            self.pool_maxsize = CFG.pool_maxsize
        
        if hasattr(CFG,'max_retries'):
            self.max_retries = CFG.max_retries
            
        if hasattr(CFG,'pool_block'):
            self.pool_block = CFG.pool_block


        """
        DATE
        """
        self.dates = bmDate().getDates()
        


    def setParam(self, **kwargs):
        """
        setParam:
        overload params with these in **kwargs.
        Params available are :
        timeInterval
        For details have a look in /etc/bmSearch/config.py
        """
        
        self.timeInterval = CFG.timeInterval
        if "timeInterval" in kwargs.keys():
            self.timeInterval = kwargs['timeInterval']

        if "headers" in kwargs.keys():
            self.params['headers'] = kwargs['headers']
        
        if "user" in kwargs.keys() and  "password" in kwargs.keys():
            from requests.auth import HTTPBasicAuth
            self.params['auth'] = HTTPBasicAuth(kwargs['user'],kwargs['password'])
        
        if "verifyPeer" in kwargs.keys():
            self.params['verify'] = kwargs['verifyPeer']
        
        if "serverCert" in kwargs.keys():
            self.params['cert'] = kwargs['serverCert']

        if "pool_connections" in kwargs.keys():
            self.pool_connections = kwargs['pool_connections']

        if "pool_maxsize" in kwargs.keys():
            self.pool_maxsize = kwargs['pool_maxsize']
            
        if "max_retries" in kwargs.keys():
            self.max_retries = kwargs['max_retries']
            
        if "pool_block" in kwargs.keys():
            self.pool_block = kwargs['pool_block']

        if "setDate" in kwargs.keys():
            d = bmDate()
            d.setDates(kwargs['setDate'])
            self.dates = d.getDates()


    def initSession(self):        
        self.session = requests.Session()
        self.adapter = requests.adapters.HTTPAdapter(pool_connections=self.pool_connections, pool_maxsize=self.pool_maxsize, max_retries=self.max_retries, pool_block=self.pool_block)

        if 'https' in self.APIURL:     
            self.session.mount('https://', self.adapter)
        else:
            self.session.mount('http://', self.adapter)
            

    def sendData(self, data):
        """
            sendData :
                Post (dict) data to an ELK platform
                data have not to be JSON formated
        """
        
        url = self.APIURL + "/bmsearch-" + self.dates['todayNumELK'] + "/_doc"
        self.params['json'] = data
        try:
            res = self.session.post(url, **self.params)
            backoffice.debug(res.text)
            return True
        except Exception as error:
            backoffice.critical(error)
            pass


    def sendDataAboutYesterday(self, data):
        """
            sendData :
                Post (dict) data to an ELK platform
                data have not to be JSON formated
        """
        
        url = self.APIURL + "/bmsearch-" + self.dates['yesterdayNumELK'] + "/_doc"
        self.params['json'] = data
        try:
            res = self.session.post(url, **self.params)
            backoffice.debug(res.text)
            return True
        except Exception as error:
            backoffice.critical(error)
            pass

    def sendDataStatus(self, data):
        """
            sendDataStatus :
                Post (dict) data to an ELK platform
                data have not to be JSON formated
                data is about all DSN lines and cyrus delivery information
        """
        
        url = self.APIURL + "/bmsearchstatus-" + self.dates['todayNumELK'] + "/_doc"
        self.params['json'] = data
        
        try:
            res = self.session.post(url, **self.params)
            backoffice.debug(res.text)
            
        except Exception as error:
            backoffice.critical(error)
            pass


 
    def sendDataLoggin(self, data):
        """
            sendDataStatus :
                Post (dict) data to an ELK platform
                data have not to be JSON formated
                data is about all cyrus loggin lines 
        """
        
        url = self.APIURL + "/bmsearchloggin-" + self.dates['todayNumELK'] + "/_doc"
        self.params['json'] = data
        
        try:
            res = self.session.post(url, **self.params)
            backoffice.debug(res.text)
        except Exception as error:
            backoffice.critical(error)
            pass



    def sendDataRoutedBy(self, data):
        """
            sendDataRoutedBy :
                Post (dict) data to an ELK platform
                data have not to be JSON formated
                data is about message-id/mail already full parsed but want keep a trace where mail was routed by and passed through
        """
        
        url = self.APIURL + "/bmsearchroutedby-" + self.dates['todayNumELK'] + "/_doc"
        self.params['json'] = data
        try:
            res = self.session.post(url, **self.params)
            backoffice.debug(res.text)
        except Exception as error:
            backoffice.critical(error)
            pass


