#!/usr/bin/env python3
# -*- coding: utf-8; py-indent-offset: 4 -*-


#  +-- bmSearch ------------------------------------------------------------------+
#  |                                                                              |
#  |  _                 _____                          _                          |
#  | | |               / ____|                        | |                         | 
#  | | |___  _ __ ___ | |____    ___  ____   _ _  ___ | |__                       |
#  | |  _  \| '_ ` _ \ \___  \  / _ \|___ \ | `_|/ __||  _  \                     | 
#  | | |_| || | | | | |____|  ||  __// __  || | | |__ | | | |                     |
#  | |_____/|_| |_| |_|______/  \___|\____/ |_|  \___||_| |_|                     |
#  |                                                                              |
#  |                                                                              |
#  +------------------------------------------------------------------------------+
#
# * ----------------------------------------------------------------------------------------*
# * @author  : Pascal SALAUN <pascal.salaun-AT-bm-monitor.org>                              *
# * @Website : http://www.bm-monitor.org <http://www.bm-monitor.org>                        *
# * @license : GNU Affero General Public License Version 3 <http://www.gnu.org/licenses>    *
# *                                                                                         *
# * bmSearch is free software;  you can redistribute it and/or modify it under the          *
# *  terms of the  license mentionned above.                                                *
# * bmSearch is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY    * 
# * ----------------------------------------------------------------------------------------*

# Native Python Lib/Class 
import os
import sqlite3
import sys

# bmSearch Class
from logger import *
backoffice = logger().setLogger().getChild('sqlQuery')

class sqlQuery:
    """
        sqlQuery : 
            This class offers all methods to manage message-id in time in datastoring them in a simple table index containing two columns ('messageid', 'timestamp') 
            This DB contain tables :
                `index` (`messageid` INTEGER NOT NULL UNIQUE, `timestamp` INTEGER NOT NULL, PRIMARY KEY(messageid) )
    """


    def __init__(self, db):
        """
            __init__ 
                init the connection to the database stored in db file
                Args :
                    "db" : the full path to the db file, by default '/var/lib/bmSearch/bmSearch.sqlite' 
        """
                        
        try:
            self.conn = sqlite3.connect(db, isolation_level='DEFERRED')
            #self.conn = sqlite3.connect(":memory:")
            self.c = self.conn.cursor()
            self.c.execute('''PRAGMA synchronous = OFF''')
            self.c.execute('''PRAGMA journal_mode = OFF''')

        except Exception as error:
            backoffice.critical(error)
            pass


    def close(self):
        """
            close :
                close the connection to the dabase
        """
        try:
            self.c.close()
            del self.c
            self.conn.close()
        
        except Exception as error:
            backoffice.critical(error)
            pass


    def initDb(self):
        """
            initDb: 
                create the sqlite database, by default '/var/lib/bmSearch/bmSearch.sqlite' 
        """
        
        try:
            self.c.execute("""
                CREATE TABLE IF NOT EXISTS `index` (
                `messageid`	TEXT NOT NULL UNIQUE,
                `timestamp`	INTEGER NOT NULL,
                PRIMARY KEY(messageid)
                );
            """)
            self.c.execute("vacuum")
            self.conn.commit()
            
            self.c.execute("""
                CREATE INDEX IF NOT EXISTS `index_idx` ON `index` (
                `messageid`,
                `timestamp`
                );
            """)
            self.c.execute("vacuum")
            self.conn.commit()
            
            self.c.execute("""
                CREATE TABLE IF NOT EXISTS `status` (
                `hashLine`	TEXT NOT NULL UNIQUE,
                `timestamp`	INTEGER NOT NULL,
                PRIMARY KEY(hashLine)
                );              
            """)
            self.c.execute("vacuum")
            self.conn.commit()          
                
            self.c.execute("""
                CREATE INDEX IF NOT EXISTS `status_idx` ON `status` (
                `hashLine`,
                `timestamp`
                );
            """)
            self.c.execute("vacuum")
            self.conn.commit()   
                
                
            self.c.execute("""                 
                CREATE TABLE IF NOT EXISTS `queueObj` (                 
                `queueid` TEXT NOT NULL UNIQUE,                 
                `queueObj` TEXT NOT NULL UNIQUE,                 
                `timestamp` INTEGER NOT NULL,                 
                PRIMARY KEY(queueid)                 
                );             
            """)             
            self.c.execute("vacuum")             
            self.conn.commit()   
            
            self.c.execute("""
                CREATE INDEX IF NOT EXISTS `queueObj_idx` ON `queueObj` (
                `queueid`,
                `timestamp`
                );
            """)
            self.c.execute("vacuum")
            self.conn.commit()   
            
        except Exception as error:
            backoffice.critical(error)
            pass



    #
    # All about index table 
    #
    
    def updateTrigger(self, dbStoreMaxDays):
        """
            updateTrigger:
                the bmSearch sqlite database has a trigger which delete entries older than dbStoreMaxDays days, 5 by default.
                this method renew the trigger with new value
        """
        dropTrigger="""
        DROP TRIGGER IF EXISTS purge_old_data ;
        """

        addTrigger="""
        CREATE TRIGGER IF NOT EXISTS purge_old_data AFTER INSERT ON `index`
        BEGIN
        DELETE FROM `index` WHERE timestamp < (strftime('%s', 'now') - """ + str(dbStoreMaxDays) + """*86400) ;
        END;
        """

        try:
            self.c.execute(dropTrigger)
            self.c.execute(addTrigger)
            self.conn.commit()   
            
        except Exception as error:
            backoffice.warning(error)
            pass


    def addEntry(self, tupl):
        """
            addEntry:
                Add an entry (messageid, timestamp) in index table
        """
        insert = "insert or replace into `index` (messageid,timestamp) values (?,?) ;"
                        
        try:
            self.c.execute(insert, tupl)
            self.conn.commit()
        except Exception as error:
            backoffice.warning(error)
            pass


    def deleteEntry(self, tupl):
        """
            deleteEntry:
                delete single entry tupl(columnName, value) in index table.
        """
        try:                        
            self.c.execute("""delete FROM index WHERE ?=?""",tupl)
            self.conn.commit()
            return 0
                                
        except Exception as error:
            backoffice.warning(error)
            pass

        
    def verifyIfMessageidExists(self, messageid):
        """
            verifyIfMessageidExists:
                return "True" or "False" if tupl(messageid) exists or not in table "index"
        """
        select = "SELECT count(*) FROM `index` WHERE messageid=\"" + messageid + "\";"

        try:
            self.c.execute(select)
            res = self.c.fetchone()

            if res[0]>0:
                return True
            else:
                return False
                        
        except Exception as error:
            backoffice.warning(error)
            pass


    def loadMessageid(self):
        """
            loadMessageid:
                retrieve all messageid from in `index` table.
        """
        messageidList = []
        
        select = "SELECT messageid FROM `index` ;"

        try:
            res = self.c.execute(select)
            for row in res:
                messageidList.append(str(row[0]))
                        
        except Exception as error:
            backoffice.warning(error)
            pass
           
        return messageidList

    
    def purgeIndexTable(self):
        """
            purgeIndexTable:
                do a truncate of index table.
        """
        try:
            self.c.execute("""truncate index ;""")
            self.c.execute("vacuum")
            self.conn.commit()
                        
        except Exception as error:
            backoffice.warning(error)
            pass


    def deleteEntriesOlderThan(self, tupl):
        """
            deleteEntriesOlderThan:
                delete all index entries older than tupl(timestamp) value
        """
        try:                                
            self.c.execute("""delete FROM index WHERE timestamp<?""",tupl)
            self.conn.commit()
            return 0
                                
        except Exception as error:
            backoffice.warning(error)
            pass




    #
    # All about status table 
    #
    
    def updateStatusTrigger(self):
        """
            updateStatusTrigger:
                the bmSearch sqlite database has a trigger which delete entries older than 1 day
        """
        dropTrigger="""
        DROP TRIGGER IF EXISTS purge_old_status ;
        """

        addTrigger="""
        CREATE TRIGGER IF NOT EXISTS purge_old_status AFTER INSERT ON `status`
        BEGIN
        DELETE FROM `status` WHERE timestamp < (strftime('%s', 'now') - 86400) ;
        END;
        """

        try:
            self.c.execute(dropTrigger)
            self.c.execute(addTrigger)
            self.conn.commit()   
            
        except Exception as error:
            backoffice.warning(error)
            pass

    
    def addStatusEntry(self, tupl):
        """
            addStatusEntry:
                 Add an entry (hashLine, timestamp) in status table
        """
        insert = "insert into `status` (hashLine,timestamp) values (?,?) ;"
                        
        try:
            self.c.execute(insert, tupl)
            self.conn.commit()
        except Exception as error:
            pass

                
    def verifyIfStatusHashLineExists(self, hashLine):        
        """
            verifyIfStatusHashLineExists:
                return "True" or "False" if tupl(hashLine) exists or not in table "status"
        """
        select = "SELECT count(*) FROM `status` WHERE hashLine=\"" + hashLine + "\";"

        try:
            self.c.execute(select)
            res = self.c.fetchone()

            if res[0]>0:
                return True
            else:
                return False
                        
        except Exception as error:
            pass


    def purgeStatusTable(self):
        """
            purgeStatusTable:
                do a truncate of status table.
        """
        try:
            self.c.execute("""truncate status ;""")
            self.c.execute("vacuum")
            self.conn.commit()
                        
        except Exception as error:
            pass      


    def loadStatusHashLine(self):
        """
            loadStatusHashLine:
                retrieve all hashLine from in `status` table.
        """
        statusHashLineList = []
        
        select = "SELECT hashLine FROM `status` ;"

        try:
            res = self.c.execute(select)
            for row in res:
                statusHashLineList.append(str(row[0]))
                        
        except Exception as error:
            backoffice.warning(error)
            pass
           
        return statusHashLineList




    #
    # All about queueObj table 
    #
    
    
    def upsertQueueObjEntry(self, tupl):
        """
            addQueueObjEntry:
                 Add an entry (queueid, queueObj, timestamp) in queueObj table
        """
        insert = "insert or replace into `queueObj` (queueid,queueObj,timestamp) values (?,?,?) ;"
                        
        try:
            self.c.execute(insert, tupl)
            self.conn.commit()
        except Exception as error:
            pass


    def getAllQueueidOlderThan(self, timestp):
        """
            getAllQueueidOlderThan:
                retrieve all line from queueObj where timestamp < timestp
                return a list 
        """
        select = "SELECT queueid FROM `queueObj` WHERE timestamp < " + timestp + ";"
 
        qlist = None
        
        try:
            qlist = [] 
            self.c.execute(select)
            res = self.c.fetchall()
            for row in res:            
                qlist.append(row[0])

        except Exception as error:
            pass
        
        return qlist


    def verifyIfQueueObjExists(self, queueid):
        """
            verifyIfQueueObjExists:
            
        """
        
        select = "SELECT count(*) FROM `queueObj` WHERE queueid=\"" + queueid + "\";"

        try:
            self.c.execute(select)
            res = self.c.fetchone()

            if res[0]>0:
                return True
            else:
                return False
                        
        except Exception as error:
            pass
     
        
    def deleteQueueObj(self, queueid):
        """
            deleteQueueObj:
            
        """
        
        deleteQueueObj = "delete from `queueObj` where queueid='" + queueid + "';"
        
        try:                        
            self.c.execute(deleteQueueObj)
            self.conn.commit()
            return 0
                                
        except Exception as error:
            pass


    def getQueueObj(self, queueid):
        """
            getQueueObj:
            
        """
        select = "SELECT queueObj FROM `queueObj` WHERE queueid = '" + queueid + "' ;"
        
        qObj = None
        
        try:
            self.c.execute(select)
            res = self.c.fetchone()
            if res[0]:
                qObj = res[0]
    
            return qObj
                         
        except Exception as error:
            pass

